# Go API client for client

Documentation for all of Ory Oathkeeper's APIs.


## Overview
This API client was generated by the [OpenAPI Generator](https://openapi-generator.tech) project.  By using the [OpenAPI-spec](https://www.openapis.org/) from a remote server, you can easily generate an API client.

- API version: 1.0.0
- Package version: 1.0.0
- Build package: org.openapitools.codegen.languages.GoClientCodegen

## Installation

Install the following dependencies:

```shell
go get github.com/stretchr/testify/assert
go get golang.org/x/oauth2
go get golang.org/x/net/context
```

Put the package under your project folder and add the following in import:

```golang
import client "github.com/ory/keto-client-go"
```

To use a proxy, set the environment variable `HTTP_PROXY`:

```golang
os.Setenv("HTTP_PROXY", "http://proxy_name:proxy_port")
```

## Configuration of Server URL

Default configuration comes with `Servers` field that contains server objects as defined in the OpenAPI specification.

### Select Server Configuration

For using other server than the one defined on index 0 set context value `sw.ContextServerIndex` of type `int`.

```golang
ctx := context.WithValue(context.Background(), client.ContextServerIndex, 1)
```

### Templated Server URL

Templated server URL is formatted using default variables from configuration or from context value `sw.ContextServerVariables` of type `map[string]string`.

```golang
ctx := context.WithValue(context.Background(), client.ContextServerVariables, map[string]string{
	"basePath": "v2",
})
```

Note, enum values are always validated and all unused variables are silently ignored.

### URLs Configuration per Operation

Each operation can use different server URL defined using `OperationServers` map in the `Configuration`.
An operation is uniquely identifield by `"{classname}Service.{nickname}"` string.
Similar rules for overriding default operation server index and variables applies by using `sw.ContextOperationServerIndices` and `sw.ContextOperationServerVariables` context maps.

```
ctx := context.WithValue(context.Background(), client.ContextOperationServerIndices, map[string]int{
	"{classname}Service.{nickname}": 2,
})
ctx = context.WithValue(context.Background(), client.ContextOperationServerVariables, map[string]map[string]string{
	"{classname}Service.{nickname}": {
		"port": "8443",
	},
})
```

## Documentation for API Endpoints

All URIs are relative to *http://localhost*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*MetadataApi* | [**GetVersion**](docs/MetadataApi.md#getversion) | **Get** /version | Return Running Software Version.
*MetadataApi* | [**IsAlive**](docs/MetadataApi.md#isalive) | **Get** /health/alive | Check HTTP Server Status
*MetadataApi* | [**IsReady**](docs/MetadataApi.md#isready) | **Get** /health/ready | Check HTTP Server and Database Status
*ReadApi* | [**GetCheck**](docs/ReadApi.md#getcheck) | **Get** /check | Check a relation tuple
*ReadApi* | [**GetExpand**](docs/ReadApi.md#getexpand) | **Get** /expand | Expand a Relation Tuple
*ReadApi* | [**GetRelationTuples**](docs/ReadApi.md#getrelationtuples) | **Get** /relation-tuples | Query relation tuples
*ReadApi* | [**PostCheck**](docs/ReadApi.md#postcheck) | **Post** /check | Check a relation tuple
*WriteApi* | [**CreateRelationTuple**](docs/WriteApi.md#createrelationtuple) | **Put** /relation-tuples | Create a Relation Tuple
*WriteApi* | [**DeleteRelationTuples**](docs/WriteApi.md#deleterelationtuples) | **Delete** /relation-tuples | Delete Relation Tuples
*WriteApi* | [**PatchRelationTuples**](docs/WriteApi.md#patchrelationtuples) | **Patch** /relation-tuples | Patch Multiple Relation Tuples


## Documentation For Models

 - [CreateRelationTupleBadRequestBody](docs/CreateRelationTupleBadRequestBody.md)
 - [CreateRelationTupleInternalServerErrorBody](docs/CreateRelationTupleInternalServerErrorBody.md)
 - [DeleteRelationTuplesBadRequestBody](docs/DeleteRelationTuplesBadRequestBody.md)
 - [DeleteRelationTuplesInternalServerErrorBody](docs/DeleteRelationTuplesInternalServerErrorBody.md)
 - [ExpandTree](docs/ExpandTree.md)
 - [GenericError](docs/GenericError.md)
 - [GetCheckBadRequestBody](docs/GetCheckBadRequestBody.md)
 - [GetCheckInternalServerErrorBody](docs/GetCheckInternalServerErrorBody.md)
 - [GetCheckResponse](docs/GetCheckResponse.md)
 - [GetExpandBadRequestBody](docs/GetExpandBadRequestBody.md)
 - [GetExpandInternalServerErrorBody](docs/GetExpandInternalServerErrorBody.md)
 - [GetExpandNotFoundBody](docs/GetExpandNotFoundBody.md)
 - [GetRelationTuplesInternalServerErrorBody](docs/GetRelationTuplesInternalServerErrorBody.md)
 - [GetRelationTuplesNotFoundBody](docs/GetRelationTuplesNotFoundBody.md)
 - [GetRelationTuplesResponse](docs/GetRelationTuplesResponse.md)
 - [HealthNotReadyStatus](docs/HealthNotReadyStatus.md)
 - [HealthStatus](docs/HealthStatus.md)
 - [InlineResponse200](docs/InlineResponse200.md)
 - [InlineResponse2001](docs/InlineResponse2001.md)
 - [InlineResponse503](docs/InlineResponse503.md)
 - [InternalRelationTuple](docs/InternalRelationTuple.md)
 - [IsInstanceAliveInternalServerErrorBody](docs/IsInstanceAliveInternalServerErrorBody.md)
 - [PatchDelta](docs/PatchDelta.md)
 - [PatchRelationTuplesBadRequestBody](docs/PatchRelationTuplesBadRequestBody.md)
 - [PatchRelationTuplesInternalServerErrorBody](docs/PatchRelationTuplesInternalServerErrorBody.md)
 - [PatchRelationTuplesNotFoundBody](docs/PatchRelationTuplesNotFoundBody.md)
 - [PostCheckBadRequestBody](docs/PostCheckBadRequestBody.md)
 - [PostCheckInternalServerErrorBody](docs/PostCheckInternalServerErrorBody.md)
 - [RelationQuery](docs/RelationQuery.md)
 - [SubjectSet](docs/SubjectSet.md)
 - [Version](docs/Version.md)


## Documentation For Authorization

 Endpoints do not require authorization.


## Documentation for Utility Methods

Due to the fact that model structure members are all pointers, this package contains
a number of utility functions to easily obtain pointers to values of basic types.
Each of these functions takes a value of the given basic type and returns a pointer to it:

* `PtrBool`
* `PtrInt`
* `PtrInt32`
* `PtrInt64`
* `PtrFloat`
* `PtrFloat32`
* `PtrFloat64`
* `PtrString`
* `PtrTime`

## Author

hi@ory.sh

